@@ -1,6 +1,6 @@
 /* 
    HTTP request/response handling
-   Copyright (C) 1999-2008, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2009, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -82,6 +82,7 @@ struct field {
 
 /* pre-calculated hash values for given header names: */
 #define HH_HV_CONNECTION        (0x14)
+#define HH_HV_PROXY_CONNECTION  (0x1A)
 #define HH_HV_CONTENT_LENGTH    (0x13)
 #define HH_HV_TRANSFER_ENCODING (0x07)
 
@@ -189,7 +190,7 @@ static int aborted(ne_request *req, const char *doing, ssize_t code)
 
     switch(code) {
     case NE_SOCK_CLOSED:
-	if (sess->use_proxy) {
+	if (sess->nexthop->proxy != PROXY_NONE) {
 	    ne_set_error(sess, _("%s: connection was closed by proxy server"),
 			 doing);
 	} else {
@@ -285,8 +286,12 @@ static ssize_t body_fd_send(void *userdata, char *buffer, size_t count)
     if (count) {
         if (req->body.file.remain == 0)
             return 0;
-        if ((off_t)count > req->body.file.remain)
-            count = req->body.file.remain;
+
+        /* Casts here are necessary for LFS platforms for safe and
+         * warning-free assignment/comparison between 32-bit size_t
+         * and 64-bit off64_t: */
+        if ((ne_off_t)count > req->body.file.remain)
+            count = (size_t)req->body.file.remain;
 	return read(req->body.file.fd, buffer, count);
     } else {
         ne_off_t newoff;
@@ -383,28 +388,36 @@ static int send_request_body(ne_request *req, int retry)
  * headers */
 static void add_fixed_headers(ne_request *req) 
 {
-    if (req->session->user_agent) {
-        ne_buffer_zappend(req->headers, req->session->user_agent);
+    ne_session *const sess = req->session;
+
+    if (sess->user_agent) {
+        ne_buffer_zappend(req->headers, sess->user_agent);
     }
 
     /* If persistent connections are disabled, just send Connection:
      * close; otherwise, send Connection: Keep-Alive to pre-1.1 origin
      * servers to try harder to get a persistent connection, except if
      * using a proxy as per 2068§19.7.1.  Always add TE: trailers. */
-    if (!req->session->flags[NE_SESSFLAG_PERSIST]) {
-       ne_buffer_czappend(req->headers,
-                          "Connection: TE, close" EOL
-                          "TE: trailers" EOL);
-    } else if (!req->session->is_http11 && !req->session->use_proxy) {
+    if (!sess->flags[NE_SESSFLAG_PERSIST]) {
+       ne_buffer_czappend(req->headers, "Connection: TE, close" EOL);
+    } 
+    else if (!sess->is_http11 && !sess->any_proxy_http) {
         ne_buffer_czappend(req->headers, 
-                          "Keep-Alive: " EOL
-                          "Connection: TE, Keep-Alive" EOL
-                          "TE: trailers" EOL);
-    } else {
+                           "Keep-Alive: " EOL
+                          "Connection: TE, Keep-Alive" EOL);
+    } 
+    else if (!req->session->is_http11 && !sess->any_proxy_http) {
         ne_buffer_czappend(req->headers, 
-                           "Connection: TE" EOL
-                           "TE: trailers" EOL);
+                           "Keep-Alive: " EOL
+                           "Proxy-Connection: Keep-Alive" EOL
+                           "Connection: TE" EOL);
+    } 
+    else {
+        ne_buffer_czappend(req->headers, "Connection: TE" EOL);
     }
+
+    ne_buffer_concat(req->headers, "TE: trailers" EOL "Host: ", 
+                     req->session->server.hostport, EOL, NULL);
 }
 
 int ne_accept_always(void *userdata, ne_request *req, const ne_status *st)
@@ -427,6 +440,8 @@ ne_request *ne_request_create(ne_session *sess,
     
     /* Presume the method is idempotent by default. */
     req->flags[NE_REQFLAG_IDEMPOTENT] = 1;
+    /* Expect-100 default follows the corresponding session flag. */
+    req->flags[NE_REQFLAG_EXPECT100] = sess->flags[NE_SESSFLAG_EXPECT100];
 
     /* Add in the fixed headers */
     add_fixed_headers(req);
@@ -435,11 +450,11 @@ ne_request *ne_request_create(ne_session *sess,
     req->method = ne_strdup(method);
     req->method_is_head = (strcmp(method, "HEAD") == 0);
 
-    /* Only use an absoluteURI here when absolutely necessary: some
-     * servers can't parse them. */
-    if (req->session->use_proxy && !req->session->use_ssl && path[0] == '/')
+    /* Only use an absoluteURI here when we might be using an HTTP
+     * proxy, and SSL is in use: some servers can't parse them. */
+    if (sess->any_proxy_http && !req->session->use_ssl && path[0] == '/')
 	req->uri = ne_concat(req->session->scheme, "://", 
-			     req->session->server.hostport, path, NULL);
+                             req->session->server.hostport, path, NULL);
     else
 	req->uri = ne_strdup(path);
 
@@ -791,14 +806,13 @@ static ne_buffer *build_request(ne_request *req)
     struct hook *hk;
     ne_buffer *buf = ne_buffer_create();
 
-    /* Add Request-Line and Host header: */
-    ne_buffer_concat(buf, req->method, " ", req->uri, " HTTP/1.1" EOL,
-		     "Host: ", req->session->server.hostport, EOL, NULL);
-    
+    /* Add Request-Line and headers: */
+    ne_buffer_concat(buf, req->method, " ", req->uri, " HTTP/1.1" EOL, NULL);
+
     /* Add custom headers: */
     ne_buffer_append(buf, req->headers->data, ne_buffer_size(req->headers));
 
-    if (req->flags[NE_REQFLAG_EXPECT100]) {
+    if (req->body_length && req->flags[NE_REQFLAG_EXPECT100]) {
         ne_buffer_czappend(buf, "Expect: 100-continue\r\n");
     }
 
@@ -1108,11 +1122,9 @@ static int read_response_headers(ne_request *req)
 }
 
 /* Perform any necessary DNS lookup for the host given by *info;
- * return NE_ code. */
+ * returns NE_ code with error string set on error. */
 static int lookup_host(ne_session *sess, struct host_info *info)
 {
-    if (sess->addrlist) return NE_OK;
-
     NE_DEBUG(NE_DBG_HTTP, "Doing DNS lookup on %s...\n", info->hostname);
     sess->status.lu.hostname = info->hostname;
     notify_status(sess, ne_status_lookup);
@@ -1137,7 +1149,7 @@ int ne_begin_request(ne_request *req)
     const ne_status *const st = &req->status;
     const char *value;
     struct hook *hk;
-    int ret;
+    int ret, forced_closure = 0;
 
     /* If a non-idempotent request is sent on a persisted connection,
      * then it is impossible to distinguish between a server failure
@@ -1190,6 +1202,7 @@ int ne_begin_request(ne_request *req)
 
             if (strcmp(token, "close") == 0) {
                 req->can_persist = 0;
+                forced_closure = 1;
             } else if (strcmp(token, "keep-alive") == 0) {
                 req->can_persist = 1;
             } else if (!req->session->is_http11
@@ -1203,6 +1216,25 @@ int ne_begin_request(ne_request *req)
         ne_free(vcopy);
     }
 
+    /* Support "Proxy-Connection: keep-alive" for compatibility with
+     * some HTTP/1.0 proxies; it is risky to do this, because an
+     * intermediary proxy may not support this HTTP/1.0 extension, but
+     * will not strip the header either.  Persistent connection
+     * support is enabled based on the presence of this header if:
+     * a) it is *necessary* to do so due to the use of a connection-auth
+     * scheme, and
+     * b) connection closure was not forced via "Connection: close".  */
+    if (req->session->nexthop->proxy == PROXY_HTTP && !req->session->is_http11
+        && !forced_closure && req->session->flags[NE_SESSFLAG_CONNAUTH]) {
+        value = get_response_header_hv(req, HH_HV_PROXY_CONNECTION,
+                                       "proxy-connection");
+        if (value && ne_strcasecmp(value, "keep-alive") == 0) {
+            NE_DEBUG(NE_DBG_HTTP, "req: Using persistent connection "
+                     "for HTTP/1.0 proxy requiring conn-auth hack.\n");
+            req->can_persist = 1;
+        }
+    }
+
     /* Decide which method determines the response message-length per
      * 2616§4.4 (multipart/byteranges is not supported): */
 
@@ -1235,8 +1267,10 @@ int ne_begin_request(ne_request *req)
     } 
     else if ((value = get_response_header_hv(req, HH_HV_CONTENT_LENGTH,
                                              "content-length")) != NULL) {
-        ne_off_t len = ne_strtoff(value, NULL, 10);
-        if (len != NE_OFFT_MAX && len >= 0) {
+        char *endptr = NULL;
+        ne_off_t len = ne_strtoff(value, &endptr, 10);
+
+        if (*value && len != NE_OFFT_MAX && len >= 0 && endptr && *endptr == '\0') {
             req->resp.mode = R_CLENGTH;
             req->resp.body.clen.total = req->resp.body.clen.remain = len;
         } else {
@@ -1399,40 +1433,32 @@ static int proxy_tunnel(ne_session *sess)
 #endif
 
 /* Return the first resolved address for the given host. */
-static const ne_inet_addr *resolve_first(ne_session *sess, 
-                                         struct host_info *host)
+static const ne_inet_addr *resolve_first(struct host_info *host)
 {
-    if (sess->addrlist) {
-        sess->curaddr = 0;
-        return sess->addrlist[0];
-    } else {
-        return ne_addr_first(host->address);
-    }
+    return host->network ? host->network : ne_addr_first(host->address);
 }
 
 /* Return the next resolved address for the given host or NULL if
  * there are no more addresses. */
-static const ne_inet_addr *resolve_next(ne_session *sess,
-                                        struct host_info *host)
+static const ne_inet_addr *resolve_next(struct host_info *host)
 {
-    if (sess->addrlist) {
-        if (sess->curaddr++ < sess->numaddrs)
-            return sess->addrlist[sess->curaddr];
-        else
-            return NULL;
-    } else {
-        return ne_addr_next(host->address);
-    }
+    return host->network ? NULL : ne_addr_next(host->address);
 }
 
 /* Make new TCP connection to server at 'host' of type 'name'.  Note
  * that once a connection to a particular network address has
  * succeeded, that address will be used first for the next attempt to
  * connect. */
-static int do_connect(ne_session *sess, struct host_info *host, const char *err)
+static int do_connect(ne_session *sess, struct host_info *host)
 {
     int ret;
 
+    /* Resolve hostname if necessary. */
+    if (host->address == NULL && host->network == NULL) {
+        ret = lookup_host(sess, host);
+        if (ret) return ret;
+    }
+
     if ((sess->socket = ne_sock_create()) == NULL) {
         ne_set_error(sess, _("Could not create socket"));
         return NE_ERROR;
@@ -1445,7 +1471,7 @@ static int do_connect(ne_session *sess, struct host_info *host, const char *err)
         ne_sock_prebind(sess->socket, sess->local_addr, 0);
 
     if (host->current == NULL)
-	host->current = resolve_first(sess, host);
+	host->current = resolve_first(host);
 
     sess->status.ci.hostname = host->hostname;
 
@@ -1461,50 +1487,140 @@ static int do_connect(ne_session *sess, struct host_info *host, const char *err)
 #endif
 	ret = ne_sock_connect(sess->socket, host->current, host->port);
     } while (ret && /* try the next address... */
-	     (host->current = resolve_next(sess, host)) != NULL);
+	     (host->current = resolve_next(host)) != NULL);
 
     if (ret) {
-        ne_set_error(sess, "%s: %s", err, ne_sock_error(sess->socket));
+        const char *msg;
+
+        if (host->proxy == PROXY_NONE)
+            msg = _("Could not connect to server");
+        else
+            msg = _("Could not connect to proxy server");
+
+        ne_set_error(sess, "%s: %s", msg, ne_sock_error(sess->socket));
         ne_sock_close(sess->socket);
-	return NE_CONNECT;
+	return ret == NE_SOCK_TIMEOUT ? NE_TIMEOUT : NE_CONNECT;
     }
 
-    notify_status(sess, ne_status_connected);
-    
     if (sess->rdtimeout)
 	ne_sock_read_timeout(sess->socket, sess->rdtimeout);
 
+    notify_status(sess, ne_status_connected);
+    sess->nexthop = host;
+
     sess->connected = 1;
     /* clear persistent connection flag. */
     sess->persisted = 0;
     return NE_OK;
 }
 
+/* For a SOCKSv4 proxy only, the IP address of the origin server (in
+ * addition to the proxy) must be known, and must be an IPv4 address.
+ * Returns NE_*; connection closed and error string set on error. */
+static int socks_origin_lookup(ne_session *sess)
+{
+    const ne_inet_addr *ia;
+    int ret;
+
+    ret = lookup_host(sess, &sess->server);
+    if (ret) {
+        /* lookup_host already set the error string. */
+        ne_close_connection(sess);
+        return ret;
+    }
+    
+    /* Find the first IPv4 address available for the server. */
+    for (ia = ne_addr_first(sess->server.address);
+         ia && ne_iaddr_typeof(ia) == ne_iaddr_ipv6;
+         ia = ne_addr_next(sess->server.address)) {
+        /* noop */
+    }
+
+    /* ... if any */
+    if (ia == NULL) {
+        ne_set_error(sess, _("Could not find IPv4 address of "
+                             "hostname %s for SOCKS v4 proxy"), 
+                     sess->server.hostname);
+        ne_close_connection(sess);
+        return NE_LOOKUP;
+    }
+
+    sess->server.current = ia;
+    
+    return ret;
+}
+
 static int open_connection(ne_session *sess) 
 {
     int ret;
-    struct host_info *host;
     
     if (sess->connected) return NE_OK;
 
-    /* Resolve hostname if necessary. */
-    host = sess->use_proxy ? &sess->proxy : &sess->server;
-    if (host->address == NULL) {
-        ret = lookup_host(sess, host);
-        if (ret) return ret;
-    }    
-    
-    ret = do_connect(sess, host, 
-                     sess->use_proxy ? 
-                     _("Could not connect to proxy server")
-                     : _("Could not connect to server"));
-    if (ret != NE_OK) return ret;
+    if (!sess->proxies) {
+        ret = do_connect(sess, &sess->server);
+        if (ret) {
+            sess->nexthop = NULL;
+            return ret;
+        }
+    }
+    else {
+        struct host_info *hi;
+
+        /* Attempt to re-use proxy to avoid iterating through
+         * unnecessarily. */
+        if (sess->prev_proxy) 
+            ret = do_connect(sess, sess->prev_proxy);
+        else
+            ret = NE_ERROR;
+
+        /* Otherwise, try everything - but omitting prev_proxy if that
+         * has already been tried. */
+        for (hi = sess->proxies; hi && ret; hi = hi->next) {
+            if (hi != sess->prev_proxy)
+                ret = do_connect(sess, hi);
+        }
+
+        if (ret == NE_OK && sess->nexthop->proxy == PROXY_SOCKS) {
+            /* Special-case for SOCKS v4 proxies, which require the
+             * client to resolve the origin server IP address. */
+            if (sess->socks_ver == NE_SOCK_SOCKSV4) {
+                ret = socks_origin_lookup(sess);
+            }
+            
+            if (ret == NE_OK) {
+                /* Perform the SOCKS handshake, instructing the proxy
+                 * to set up the connection to the origin server. */
+                ret = ne_sock_proxy(sess->socket, sess->socks_ver, 
+                                    sess->server.current,
+                                    sess->server.hostname, sess->server.port,
+                                    sess->socks_user, sess->socks_password);
+                if (ret) {
+                    ne_set_error(sess, 
+                                 _("Could not establish connection from "
+                                   "SOCKS proxy (%s:%u): %s"),
+                                 sess->nexthop->hostname,
+                                 sess->nexthop->port,
+                                 ne_sock_error(sess->socket));
+                    ne_close_connection(sess);
+                }
+            }
+        }
+
+        if (ret != NE_OK) {
+            sess->nexthop = NULL;
+            sess->prev_proxy = NULL;
+            return ret;
+        }
+        
+        /* Success - make this proxy stick. */
+        sess->prev_proxy = hi;
+    }
 
 #ifdef NE_HAVE_SSL
     /* Negotiate SSL layer if required. */
     if (sess->use_ssl && !sess->in_connect) {
-        /* CONNECT tunnel */
-        if (sess->use_proxy)
+        /* Set up CONNECT tunnel if using an HTTP proxy. */
+        if (sess->nexthop->proxy == PROXY_HTTP)
             ret = proxy_tunnel(sess);
         
         if (ret == NE_OK) {
