@@ -1,7 +1,6 @@
 /* 
    Socket handling routines
-   Copyright (C) 1998-2008, Joe Orton <joe@manyfish.co.uk>, 
-   Copyright (C) 1999-2000 Tommi Komulainen <Tommi.Komulainen@iki.fi>
+   Copyright (C) 1998-2009, Joe Orton <joe@manyfish.co.uk>
    Copyright (C) 2004 Aleix Conchillo Flaque <aleix@member.fsf.org>
 
    This library is free software; you can redistribute it and/or
@@ -60,6 +59,7 @@
 #include <stddef.h>
 #ifdef USE_GETADDRINFO
 #include <ws2tcpip.h>
+#include <wspiapi.h>
 #endif
 #endif
 
@@ -116,9 +116,7 @@ typedef struct addrinfo ne_inet_addr;
 typedef struct in_addr ne_inet_addr;
 #endif
 
-#ifdef NE_HAVE_SSL
 #include "ne_privssl.h" /* MUST come after ne_inet_addr is defined */
-#endif
 
 /* To avoid doing AAAA queries unless absolutely necessary, either use
  * AI_ADDRCONFIG where available, or a run-time check for working IPv6
@@ -190,6 +188,10 @@ struct iofns {
     /* Wait up to 'n' seconds for socket to become readable.  Returns
      * 0 when readable, otherwise NE_SOCK_TIMEOUT or NE_SOCK_ERROR. */
     int (*readable)(ne_socket *s, int n);
+    /* Write up to 'count' blocks described by 'vector' to socket.
+     * Return number of bytes written on success, or <0 on error. */
+    ssize_t (*swritev)(ne_socket *s, const struct ne_iovec *vector, 
+                       int count);
 };
 
 static const ne_inet_addr dummy_laddr;
@@ -293,8 +295,10 @@ static void init_ipv6(void)
     else
         close(fd);
 }
-#else
+#elif defined(AF_INET6)
 #define ipv6_disabled (0)
+#else
+#define ipv6_disabled (1)
 #endif
 
 /* If init_state is N where > 0, ne_sock_init has been called N times;
@@ -394,10 +398,15 @@ static int raw_poll(int fdno, int rdwr, int secs)
     /* Init the fd set */
     FD_ZERO(&rdfds);
     FD_ZERO(&wrfds);
-    if (rdwr == 0)
+
+    /* Note that (amazingly) the FD_SET macro does not expand
+     * correctly on Netware if not inside a compound statement
+     * block. */
+    if (rdwr == 0) {
         FD_SET(fdno, &rdfds);
-    else
+    } else {
         FD_SET(fdno, &wrfds);
+    }
 
     if (tvp) {
         tvp->tv_sec = secs;
@@ -540,7 +549,49 @@ static ssize_t write_raw(ne_socket *sock, const char *data, size_t length)
     return ret;
 }
 
-static const struct iofns iofns_raw = { read_raw, write_raw, readable_raw };
+static ssize_t writev_raw(ne_socket *sock, const struct ne_iovec *vector, int count) 
+{
+    ssize_t ret;
+#ifdef WIN32
+    LPWSABUF wasvector = (LPWSABUF)ne_malloc(count * sizeof(WSABUF));
+    DWORD total;
+    int i;
+
+    for (i = 0; i < count; i++){
+        wasvector[i].buf = vector[i].base;
+        wasvector[i].len = vector[i].len;
+    }
+        
+    ret = WSASend(sock->fd, wasvector, count, &total, 0, NULL, NULL);
+    if (ret == 0)
+        ret = total;
+    
+    ne_free(wasvector);
+#else
+    const struct iovec *vec = (const struct iovec *) vector;
+
+    do {
+	ret = writev(sock->fd, vec, count);
+    } while (ret == -1 && NE_ISINTR(ne_errno));
+#endif
+
+    if (ret < 0) {
+	int errnum = ne_errno;
+	set_strerror(sock, errnum);
+	return MAP_ERR(errnum);
+    }
+    
+    return ret;
+}
+
+#ifdef NE_HAVE_SSL
+static ssize_t writev_dummy(ne_socket *sock, const struct ne_iovec *vector, int count) 
+{
+    return sock->ops->swrite(sock, vector[0].base, vector[0].len);
+}
+#endif
+
+static const struct iofns iofns_raw = { read_raw, write_raw, readable_raw, writev_raw };
 
 #ifdef HAVE_OPENSSL
 /* OpenSSL I/O function implementations. */
@@ -624,7 +675,8 @@ static ssize_t write_ossl(ne_socket *sock, const char *data, size_t len)
 static const struct iofns iofns_ssl = {
     read_ossl,
     write_ossl,
-    readable_ossl
+    readable_ossl,
+    writev_dummy
 };
 
 #elif defined(HAVE_GNUTLS)
@@ -698,13 +750,18 @@ static ssize_t error_gnutls(ne_socket *sock, ssize_t sret)
 static ssize_t read_gnutls(ne_socket *sock, char *buffer, size_t len)
 {
     ssize_t ret;
+    unsigned reneg = 1; /* number of allowed rehandshakes */
 
     ret = readable_gnutls(sock, sock->rdtimeout);
     if (ret) return ret;
     
     do {
-        ret = gnutls_record_recv(sock->ssl, buffer, len);
-    } while (RETRY_GNUTLS(sock, ret));
+        do {
+            ret = gnutls_record_recv(sock->ssl, buffer, len);
+        } while (RETRY_GNUTLS(sock, ret));
+        
+    } while (ret == GNUTLS_E_REHANDSHAKE && reneg--
+             && (ret = gnutls_handshake(sock->ssl)) == GNUTLS_E_SUCCESS);
 
     if (ret <= 0)
 	ret = error_gnutls(sock, ret);
@@ -729,7 +786,8 @@ static ssize_t write_gnutls(ne_socket *sock, const char *data, size_t len)
 static const struct iofns iofns_ssl = {
     read_gnutls,
     write_gnutls,
-    readable_gnutls
+    readable_gnutls,
+    writev_dummy
 };
 
 #endif
@@ -749,6 +807,32 @@ int ne_sock_fullwrite(ne_socket *sock, const char *data, size_t len)
     return ret < 0 ? ret : 0;
 }
 
+int ne_sock_fullwritev(ne_socket *sock, const struct ne_iovec *vector, int count)
+{
+    ssize_t ret;
+
+    do {
+        ret = sock->ops->swritev(sock, vector, count);
+        if (ret > 0) {
+            while (count && (size_t)ret >= vector[0].len) {
+                ret -= vector[0].len;
+                count--;
+                vector++;
+            }
+            
+            if (ret && count) {
+                /* Partial buffer sent; send the rest. */
+                ret = ne_sock_fullwrite(sock, (char *)vector[0].base + ret,
+                                        vector[0].len - ret);
+                count--;
+                vector++;
+            }
+        }
+    } while (count && ret >= 0);
+
+    return ret < 0 ? ret : 0;
+}
+
 ssize_t ne_sock_readline(ne_socket *sock, char *buf, size_t buflen)
 {
     char *lf;
@@ -823,7 +907,10 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
 #ifdef USE_GETADDRINFO
     struct addrinfo hints = {0};
     char *pnt;
+
     hints.ai_socktype = SOCK_STREAM;
+
+#ifdef AF_INET6
     if (hostname[0] == '[' && ((pnt = strchr(hostname, ']')) != NULL)) {
 	char *hn = ne_strdup(hostname + 1);
 	hn[pnt - hostname - 1] = '\0';
@@ -833,7 +920,9 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
         hints.ai_family = AF_INET6;
 	addr->errnum = getaddrinfo(hn, NULL, &hints, &addr->result);
 	ne_free(hn);
-    } else {
+    } else 
+#endif /* AF_INET6 */
+    {
 #ifdef USE_GAI_ADDRCONFIG /* added in the RFC3493 API */
         hints.ai_flags = AI_ADDRCONFIG;
         hints.ai_family = AF_UNSPEC;
@@ -961,6 +1050,83 @@ char *ne_iaddr_print(const ne_inet_addr *ia, char *buf, size_t bufsiz)
     return buf;
 }
 
+unsigned char *ne_iaddr_raw(const ne_inet_addr *ia, unsigned char *buf)
+{
+#ifdef USE_GETADDRINFO
+#ifdef AF_INET6
+    if (ia->ai_family == AF_INET6) {
+	struct sockaddr_in6 *in6 = SACAST(in6, ia->ai_addr);
+        return memcpy(buf, in6->sin6_addr.s6_addr, sizeof in6->sin6_addr.s6_addr);
+    } else
+#endif /* AF_INET6 */
+    {
+	struct sockaddr_in *in = SACAST(in, ia->ai_addr);
+        return memcpy(buf, &in->sin_addr.s_addr, sizeof in->sin_addr.s_addr);
+    }
+#else /* !USE_GETADDRINFO */
+    return memcpy(buf, &ia->s_addr, sizeof ia->s_addr);
+#endif
+}
+
+ne_inet_addr *ne_iaddr_parse(const char *addr, ne_iaddr_type type)
+{
+#if defined(USE_GETADDRINFO) && defined(HAVE_INET_PTON)
+    char dst[sizeof(struct in6_addr)];
+    int af = type == ne_iaddr_ipv6 ? AF_INET6 : AF_INET;
+
+    if (inet_pton(af, addr, dst) != 1) {
+        return NULL;
+    }
+    
+    return ne_iaddr_make(type, (unsigned char *)dst);
+#elif defined(USE_GETADDRINFO) && !defined(HAVE_INET_PTON)
+    /* For Windows, which lacks inet_pton(). */
+    struct addrinfo *ai, *rv, hints;
+
+    memset(&hints, 0, sizeof hints);
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_NUMERICHOST;
+    hints.ai_family = type == ne_iaddr_ipv6 ? AF_INET6 : AF_INET;
+    
+    if (getaddrinfo(addr, NULL, &hints, &ai)) {
+        return NULL;
+    }
+    
+    /* Copy the returned addrinfo, since it needs to be ne_free()-able
+     * later; must only call freeaddrinfo() on ai. */
+    rv = ne_calloc(sizeof *rv);
+    memcpy(rv, ai, sizeof *rv);
+    rv->ai_next = NULL;
+    rv->ai_canonname = NULL;
+    rv->ai_addr = ne_calloc(ai->ai_addrlen);
+    memcpy(rv->ai_addr, ai->ai_addr, ai->ai_addrlen);
+    freeaddrinfo(ai);
+    
+    return rv;    
+#else /* !USE_GETADDRINFO */
+    struct in_addr a;
+    
+    if (type == ne_iaddr_ipv6) {
+        return NULL;
+    }
+
+#ifdef WIN32
+    /* inet_addr() is broken because INADDR_NONE is a valid
+     * broadcast address, so only use it on Windows. */
+    a.s_addr = inet_addr(addr);
+    if (a.s_addr == INADDR_NONE) {
+        return NULL;
+    }
+#else /* !WIN32 */
+    if (inet_aton(addr, &a) == 0) {
+        return NULL;
+    }
+#endif
+    
+    return ne_iaddr_make(ne_iaddr_ipv4, (unsigned char *)&a.s_addr);
+#endif /* !USE_GETADDRINFO */
+}
+
 int ne_iaddr_reverse(const ne_inet_addr *ia, char *buf, size_t bufsiz)
 {
 #ifdef USE_GETADDRINFO
@@ -990,6 +1156,20 @@ void ne_addr_destroy(ne_sock_addr *addr)
     ne_free(addr);
 }
 
+/* Perform a connect() for given fd, handling EINTR retries.  Returns
+ * zero on success or -1 on failure, in which case, ne_errno is set
+ * appropriately. */
+static int raw_connect(int fd, const struct sockaddr *sa, size_t salen)
+{
+    int ret;
+
+    do {
+        ret = connect(fd, sa, salen);
+    } while (ret < 0 && NE_ISINTR(ne_errno));
+
+    return ret;
+}
+
 /* Perform a connect() for fd to address sa of length salen, with a
  * timeout if supported on this platform.  Returns zero on success or
  * NE_SOCK_* on failure, with sock->error set appropriately. */
@@ -1004,12 +1184,17 @@ static int timed_connect(ne_socket *sock, int fd,
 
         /* Get flags and then set O_NONBLOCK. */
         flags = fcntl(fd, F_GETFL);
-        if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
+        if (flags & O_NONBLOCK) {
+            /* This socket was created using SOCK_NONBLOCK... flip the
+             * bit for restoring flags later. */
+            flags &= ~O_NONBLOCK;
+        }
+        else if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
             set_strerror(sock, errno);
             return NE_SOCK_ERROR;
         }
         
-        ret = connect(fd, sa, salen);
+        ret = raw_connect(fd, sa, salen);
         if (ret == -1) {
             errnum = ne_errno;
             if (NE_ISINPROGRESS(errnum)) {
@@ -1053,7 +1238,7 @@ static int timed_connect(ne_socket *sock, int fd,
     } else 
 #endif /* USE_NONBLOCKING_CONNECT */
     {
-        ret = connect(fd, sa, salen);
+        ret = raw_connect(fd, sa, salen);
         
         if (ret < 0) {
             set_strerror(sock, errno);
@@ -1142,7 +1327,7 @@ static int do_bind(int fd, int peer_family,
 #endif        
     
 
-#ifdef USE_GETADDRINFO
+#if defined(USE_GETADDRINFO) && defined(AF_INET6)
     /* Use a sockaddr_in6 if an AF_INET6 local address is specifed, or
      * if no address is specified and the peer address is AF_INET6: */
     if ((addr != &dummy_laddr && addr->ai_family == AF_INET6)
@@ -1179,14 +1364,43 @@ static int do_bind(int fd, int peer_family,
     }
 }
 
+#ifdef SOCK_CLOEXEC
+/* sock_cloexec is initialized to SOCK_CLOEXEC and cleared to zero if
+ * a socket() call ever fails with EINVAL. */
+static int sock_cloexec = SOCK_CLOEXEC;
+#define RETRY_ON_EINVAL
+#else
+#define sock_cloexec 0
+#endif
+
 int ne_sock_connect(ne_socket *sock,
                     const ne_inet_addr *addr, unsigned int port)
 {
     int fd, ret;
+    int type = SOCK_STREAM | sock_cloexec;
+
+#if defined(RETRY_ON_EINVAL) && defined(SOCK_NONBLOCK) \
+    && defined(USE_NONBLOCKING_CONNECT)
+    /* If the SOCK_NONBLOCK flag is defined, and the retry-on-EINVAL
+     * logic is enabled, and the socket has a configured timeout, then
+     * also use the SOCK_NONBLOCK flag to save enabling O_NONBLOCK
+     * later. */
+    if (sock->cotimeout && sock_cloexec) {
+        type |= SOCK_NONBLOCK;
+    }
+#endif
 
     /* use SOCK_STREAM rather than ai_socktype: some getaddrinfo
      * implementations do not set ai_socktype, e.g. RHL6.2. */
-    fd = socket(ia_family(addr), SOCK_STREAM, ia_proto(addr));
+    fd = socket(ia_family(addr), type, ia_proto(addr));
+#ifdef RETRY_ON_EINVAL
+    /* Handle forwards compat for new glibc on an older kernels; clear
+     * the sock_cloexec flag and retry the call: */
+    if (fd < 0 && sock_cloexec && errno == EINVAL) {
+        sock_cloexec = 0;
+        fd = socket(ia_family(addr), SOCK_STREAM, ia_proto(addr));
+    }
+#endif
     if (fd < 0) {
         set_strerror(sock, ne_errno);
 	return -1;
@@ -1201,9 +1415,10 @@ int ne_sock_connect(ne_socket *sock,
 #endif
    
 #if defined(HAVE_FCNTL) && defined(F_GETFD) && defined(F_SETFD) \
-    && defined(FD_CLOEXEC)
-    /* Set the FD_CLOEXEC bit for the new fd. */
-    if ((ret = fcntl(fd, F_GETFD)) >= 0) {
+  && defined(FD_CLOEXEC)
+    /* Set the FD_CLOEXEC bit for the new fd, if the socket was not
+     * created with the CLOEXEC bit already set. */
+    if (!sock_cloexec && (ret = fcntl(fd, F_GETFD)) >= 0) {
         fcntl(fd, F_SETFD, ret | FD_CLOEXEC);
         /* ignore failure; not a critical error. */
     }
@@ -1220,9 +1435,8 @@ int ne_sock_connect(ne_socket *sock,
         }
     }
 
-#if defined(TCP_NODELAY) && defined(HAVE_SETSOCKOPT) && defined(IPPROTO_TCP)
-    { /* Disable the Nagle algorithm; better to add write buffering
-       * instead of doing this. */
+#if defined(HAVE_SETSOCKOPT) && (defined(TCP_NODELAY) || defined(WIN32))
+    { /* Disable the Nagle algorithm. */
         int flag = 1;
         setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &flag, sizeof flag);
     }
@@ -1240,8 +1454,9 @@ int ne_sock_connect(ne_socket *sock,
 ne_inet_addr *ne_sock_peer(ne_socket *sock, unsigned int *port)
 {
     union saun {
+        struct sockaddr sa;
         struct sockaddr_in sin;
-#ifdef USE_GETADDRINFO
+#if defined(USE_GETADDRINFO) && defined(AF_INET6)
         struct sockaddr_in6 sin6;
 #endif
     } saun;
@@ -1254,18 +1469,25 @@ ne_inet_addr *ne_sock_peer(ne_socket *sock, unsigned int *port)
         return NULL;
     }
 
+#if !defined(USE_GETADDRINFO) || !defined(AF_INET6)
+    if (sad->sa_family != AF_INET) {
+        set_error(sock, _("Socket family not supported"));
+        return NULL;
+    }
+#endif                  
+
     ia = ne_calloc(sizeof *ia);
 #ifdef USE_GETADDRINFO
     ia->ai_addr = ne_malloc(sizeof *ia);
     ia->ai_addrlen = len;
     memcpy(ia->ai_addr, sad, len);
-    ia->ai_family = sad->sa_family;
+    ia->ai_family = saun.sa.sa_family;
 #else
     memcpy(ia, &saun.sin.sin_addr.s_addr, sizeof *ia);
 #endif    
 
-#ifdef USE_GETADDRINFO
-    *port = ntohs(sad->sa_family == AF_INET ? 
+#if defined(USE_GETADDRINFO) && defined(AF_INET6)
+    *port = ntohs(saun.sa.sa_family == AF_INET ? 
                   saun.sin.sin_port : saun.sin6.sin6_port);
 #else
     *port = ntohs(saun.sin.sin_port);
@@ -1312,7 +1534,7 @@ ne_inet_addr *ne_iaddr_make(ne_iaddr_type type, const unsigned char *raw)
 
 ne_iaddr_type ne_iaddr_typeof(const ne_inet_addr *ia)
 {
-#ifdef USE_GETADDRINFO
+#if defined(USE_GETADDRINFO) && defined(AF_INET6)
     return ia->ai_family == AF_INET6 ? ne_iaddr_ipv6 : ne_iaddr_ipv4;
 #else
     return ne_iaddr_ipv4;
@@ -1329,16 +1551,20 @@ int ne_iaddr_cmp(const ne_inet_addr *i1, const ne_inet_addr *i2)
 	    *in2 = SACAST(in, i2->ai_addr);
 	return memcmp(&in1->sin_addr.s_addr, &in2->sin_addr.s_addr, 
 		      sizeof in1->sin_addr.s_addr);
-    } else if (i1->ai_family == AF_INET6) {
+    } 
+#ifdef AF_INET6
+    else if (i1->ai_family == AF_INET6) {
 	struct sockaddr_in6 *in1 = SACAST(in6, i1->ai_addr), 
 	    *in2 = SACAST(in6, i2->ai_addr);
 	return memcmp(in1->sin6_addr.s6_addr, in2->sin6_addr.s6_addr,
 		      sizeof in1->sin6_addr.s6_addr);
-    } else
+    } 
+#endif /* AF_INET6 */
+    else
 	return -1;
 #else
     return memcmp(&i1->s_addr, &i2->s_addr, sizeof i1->s_addr);
-#endif
+#endif /* USE_GETADDRINFO */
 }
 
 void ne_iaddr_free(ne_inet_addr *addr)
@@ -1447,6 +1673,10 @@ int ne_sock_accept_ssl(ne_socket *sock, ne_ssl_context *ctx)
     if (ret != 1) {
         return error_ossl(sock, ret);
     }
+
+    if (SSL_session_reused(ssl)) {
+        NE_DEBUG(NE_DBG_SSL, "ssl: Server reused session.\n");
+    }
 #elif defined(HAVE_GNUTLS)
     gnutls_init(&ssl, GNUTLS_SERVER);
     gnutls_credentials_set(ssl, GNUTLS_CRD_CERTIFICATE, ctx->cred);
@@ -1651,6 +1881,15 @@ const char *ne_sock_error(const ne_socket *sock)
     return sock->error;
 }
 
+void ne_sock_set_error(ne_socket *sock, const char *format, ...)
+{
+    va_list params;
+
+    va_start(params, format);
+    ne_vsnprintf(sock->error, sizeof sock->error, format, params);
+    va_end(params);
+}
+
 /* Closes given ne_socket */
 int ne_sock_close(ne_socket *sock)
 {
@@ -1658,7 +1897,12 @@ int ne_sock_close(ne_socket *sock)
 
 #if defined(HAVE_OPENSSL)
     if (sock->ssl) {
-	SSL_shutdown(sock->ssl);
+        /* Correct SSL shutdown procedure: call once... */
+        if (SSL_shutdown(sock->ssl) == 0) {
+            /* close_notify sent but not received; wait for peer to
+             * send close_notify... */
+            SSL_shutdown(sock->ssl);
+        }
 	SSL_free(sock->ssl);
     }
 #elif defined(HAVE_GNUTLS)
