@@ -82,10 +82,12 @@
 static netrc_entry *netrc_list; /* list of netrc entries */
 #endif
 
+/* Module-private state: */
 const char *lock_store_fn = NULL;
-
 static char *progname; /* argv[0] */
+static char *rcfile;
 
+/* Global state: */
 char *proxy_hostname;
 int proxy_port;
 char *server_username = NULL, *server_password = NULL;
@@ -95,8 +97,6 @@ struct session session;
 
 ne_ssl_client_cert *client_cert;
 
-/* Global options */
-
 int tolerant; /* tolerate DAV-enabledness failure */
 
 /* Current output state */
@@ -128,9 +128,11 @@ static void usage(void)
 "Usage: %s [OPTIONS] http://hostname[:port]/path\n"
 "  Port defaults to 80, path defaults to '/'\n"
 "Options:\n"
-"  -t, --tolerant   Allow cd/open into non-WebDAV enabled collection.\n"
-"  -V, --version    Display version information.\n"
-"  -h, --help       Display this help message.\n"
+"  -t, --tolerant            Allow cd/open into non-WebDAV enabled collection.\n"
+"  -r, --rcfile=FILE         Read script from FILE instead of ~/.cadaverrc.\n"
+"  -p, --proxy=PROXY[:PORT]  Use proxy host PROXY and optional proxy port PORT.\n"
+"  -V, --version             Display version information.\n"
+"  -h, --help                Display this help message.\n"
 "Please send bug reports and feature requests to <cadaver@webdav.org>\n"), progname);
 }
 
@@ -376,6 +378,9 @@ void open_connection(const char *url)
 	}
     }
 #endif /* ENABLE_NETRC */
+#ifdef NE_SESSFLAG_EXPECT100
+	ne_set_session_flag(session.sess, NE_SESSFLAG_EXPECT100, get_bool_option(opt_expect100));
+#endif /* NE_SESSFLAG_EXPECT100 */
     session.connected = 0;
 
     ne_set_useragent(session.sess, "cadaver/" PACKAGE_VERSION);
@@ -435,15 +440,17 @@ static void parse_args(int argc, char **argv)
 	{ "help", no_argument, NULL, 'h' },
 	{ "proxy", required_argument, NULL, 'p' },
 	{ "tolerant", no_argument, NULL, 't' },
+	{ "rcfile", required_argument, NULL, 'r' },
 	{ 0, 0, 0, 0 }
     };
     int optc;
-    while ((optc = getopt_long(argc, argv, "ehtp:V", opts, NULL)) != -1) {
+    while ((optc = getopt_long(argc, argv, "ehtp:r:V", opts, NULL)) != -1) {
 	switch (optc) {
 	case 'h': usage(); exit(-1);
 	case 'V': execute_about(); exit(-1);
 	case 'p': set_proxy(optarg); break;
 	case 't': tolerant = 1; break;
+	case 'r': rcfile = strdup(optarg); break;
 	case '?': 
 	default:
 	    printf(_("Try `%s --help' for more information.\n"), progname);
@@ -586,31 +593,41 @@ static void init_netrc(void)
 #endif
 }
 
-static void init_rcfile(void)
+static int init_rcfile(void)
 {
-    char *rcfile, buf[BUFSIZ];
+    int ret = 0;
+    char buf[BUFSIZ];
     struct stat st;
     FILE *f;
-    rcfile = ne_concat(getenv("HOME"), "/.cadaverrc", NULL);
-    if (stat(rcfile, &st) != 0) {
-	NE_DEBUG(DEBUG_FILES, "No rcfile.\n");
-    } else {
-	f = fopen(rcfile, "r");
-	if (f == NULL) {
-	    printf(_("Could not read rcfile %s: %s\n"), rcfile, 
-		   strerror(errno));
-	} else {
-	    for (;;) {
-		if (fgets(buf, BUFSIZ, f) != NULL) {
-		    execute_command(ne_shave(buf, "\r\n"));
-		} else {
-		    break;
-		}
-	    }
-	    fclose(f);
+
+    if (rcfile == NULL) {
+	rcfile = ne_concat(getenv("HOME"), "/.cadaverrc", NULL);
+
+	if (stat(rcfile, &st) != 0) {
+	    NE_DEBUG(DEBUG_FILES, "No rcfile\n");
+	    free(rcfile);
+	    return 0;
 	}
     }
+
+    f = fopen(rcfile, "r");
+    if (f == NULL) {
+        printf(_("Could not read rcfile %s: %s\n"), rcfile, 
+	   strerror(errno));
+    } else {
+	for (;;) {
+	    if (fgets(buf, BUFSIZ, f) != NULL) {
+		ret = execute_command(ne_shave(buf, "\r\n"));
+		if (ret != 0)
+		    break;
+	    } else {
+		break;
+	    }
+	}
+	fclose(f);
+    }
     free(rcfile);
+    return ret;
 }
 
 
@@ -867,10 +884,11 @@ int main(int argc, char *argv[])
 
     init_signals();
     init_locking();
-    init_rcfile();
     
     parse_args(argc, argv);
 
+    ret = init_rcfile();
+
     init_readline();
 
     while (ret == 0) {
@@ -1046,7 +1064,7 @@ static void pretty_progress_bar(ne_off_t progress, ne_off_t total)
 	putchar((n<len-1)?'=':
 		 (n==(len-1)?'>':' '));
     }
-    printf(_("] %5.1f%% of %" NE_FMT_OFF_T " bytes"), pc*100, total);
+    printf(_("] %5.1f%% of %" NE_FMT_NE_OFF_T " bytes"), pc*100, total);
     fflush(stdout);
 }
 
