@@ -550,14 +550,16 @@ static void execute_mkcol(const char *name)
 static int all_iterator(void *userdata, const ne_propname *pname,
 			 const char *value, const ne_status *status)
 {
+    char *dnspace = utf8_decode(pname->nspace);
     char *dname = utf8_decode(pname->name);
     if (value != NULL) {
 	char *dval = utf8_decode(value);
-	printf("%s = %s\n", dname, dval);
+	printf("%s %s = %s\n", dnspace, dname, dval);
 	free(dval);
     } else if (status != NULL) {
-	printf(_("%s failed: %s\n"), dname, status->reason_phrase);
+	printf(_("%s %s failed: %s\n"), dnspace, dname, status->reason_phrase);
     }
+    free(dnspace);
     free(dname);
     return 0;
 }
@@ -965,7 +967,7 @@ static void execute_get(const char *remote, const char *local)
 	filename = ne_strdup(local);
     }
     {
-	int fd = open(filename, O_CREAT|O_WRONLY|O_TRUNC|OPEN_BINARY_FLAGS, 
+	int fd = open(filename, O_CREAT|O_WRONLY|O_TRUNC|OPEN_BINARY_FLAGS|O_LARGEFILE, 
                       0644);
 	output(o_download, _("Downloading `%s' to %s:"), real_remote, filename);
 	if (fd < 0) {
@@ -987,7 +989,7 @@ static void execute_get(const char *remote, const char *local)
 
 static void simple_put(const char *local, const char *remote)
 {
-    int fd = open(local, O_RDONLY | OPEN_BINARY_FLAGS);
+    int fd = open(local, O_RDONLY | OPEN_BINARY_FLAGS | O_LARGEFILE);
     output(o_upload, _("Uploading %s to `%s':"), local, remote);
     if (fd < 0) {
 	output(o_finish, _("Could not open file: %s\n"), strerror(errno));
