@@ -40,7 +40,7 @@ const struct command *get_command(const char *name);
 char *resolve_path(const char *dir, const char *filename, int isdir);
 
 /* Displays cadaver version details. */
-void execute_version(void);
+void execute_about(void);
 
 /* Returns owner href. */
 char *getowner(void);
