@@ -52,14 +52,21 @@ static void set_lockdepth(const char *new);
 static void unset_lockdepth(const char *new);
 static void disp_lockdepth(void);
 
+static void set_searchdepth(const char *new);
+static void unset_searchdepth(const char *new);
+static void disp_searchdepth(void);
+
 /* Option holders */
 
-static int enable_expect, presume_utf8, overwrite, quiet;
+static int enable_expect, presume_utf8, overwrite, quiet, searchall;
 
 enum ne_lock_scope lockscope;
 int lockdepth;
 
-static struct {
+/* search option global */
+int searchdepth;
+
+static struct option {
     const char *name;
     enum option_id id;
     void *holder; /* for bool + string options */
@@ -81,19 +88,25 @@ static struct {
     /* Booleans */
     B(tolerant, &tolerant, "Tolerate non-WebDAV collections"),
     B(overwrite, &overwrite, "Enable overwrite (e.g. on copy/move operations"),
+#ifdef NE_SESSFLAG_EXPECT100
     B(expect100, &enable_expect, "Enable use of 'Expect: 100-continue' header"),
+#endif /* NE_SESSFLAG_EXPECT100 */
     B(utf8, &presume_utf8, "Presume filenames etc are UTF-8 encoded"),
     B(quiet, &quiet, "Whether to display connection status messages"),
+    B(searchall, &searchall, "Whether to search and display all props including dead props"),
 #undef B
-#define S(x,h) { #x, opt_##x, NULL, opt_string, NULL, NULL, NULL, h, NULL }
+#define S2(x,y, h) { x, y, NULL, opt_string, NULL, NULL, NULL, h, NULL }
+#define S(x,h) S2(#x, opt_##x, h)
     S(lockowner, "Lock owner URI"),
     S(lockstore, "Persistent lock storage file"),
     S(editor, "Editor to use with `edit' command"),
-    S(cert, "Client certificate to use for SSL connections."),
-    S(certkey, "Private key for client certificate"),
+    S2("client-cert", opt_clicert,
+       "Client certificate to use for SSL connections."),
     S(namespace, "Namespace to use for propset/propget commands."),
     S(pager, "Command to run for less/more commands."),
     S(proxy, "Hostname of proxy server"),
+    S(searchorder,  "Search ascending props options"),
+    S(searchdorder, "Search descending props options"),
     { "proxy-port", opt_proxy_port, NULL, opt_string, NULL, NULL, NULL,
       "Port to use on proxy server", NULL },
 #undef S
@@ -110,6 +123,13 @@ static struct {
       set_lockdepth, unset_lockdepth, disp_lockdepth, "Lock depth options",
       "The lockdepth value must be 0 or infinity."
     },
+
+    /* Several options for search */
+    { "searchdepth", opt_searchdepth, NULL, opt_handled,
+      set_searchdepth, unset_searchdepth, disp_searchdepth, "Search depth options",
+      "The searchdepth value must be 0, 1 or infinity."
+    },
+
     { NULL, 0 }
 };
 
@@ -130,6 +150,18 @@ static const struct {
     { NULL, 0 }
 };
 
+#ifdef NE_SESSFLAG_EXPECT100
+static void setunset_expect100(void)
+{
+    if (session.connected)
+	return;
+
+    ne_set_session_flag(session.sess, NE_SESSFLAG_EXPECT100, get_bool_option(opt_expect100));
+}
+#else
+#define setunset_expect100() {}
+#endif /* NE_SESSFLAG_EXPECT100 */
+
 static void display_options(void) 
 {
     int n;
@@ -138,18 +170,18 @@ static void display_options(void)
 	int *val = (int *)options[n].holder;
 	switch (options[n].type) {
 	case opt_bool:
-	    printf(" %10s: %s\n", options[n].name, *val?"on":"off");
+	    printf(" %15s: %s\n", options[n].name, *val?"on":"off");
 	    break;
 	case opt_string:
 	    if (options[n].holder == NULL) {
-		printf(" %10s: unset\n", options[n].name);
+		printf(" %15s: unset\n", options[n].name);
 	    } else {
-		printf(" %10s: %s\n", options[n].name, 
+		printf(" %15s: %s\n", options[n].name, 
 		       (char *)options[n].holder);
 	    }
 	    break;
 	case opt_handled:
-	    printf(" %10s: ", options[n].name);
+	    printf(" %15s: ", options[n].name);
 	    (*options[n].display)();
 	    printf("\n");
 	    break;
@@ -265,6 +297,45 @@ static void disp_lockdepth(void)
 	printf("illegal value");
 }
 
+
+static void set_searchdepth(const char *set)
+{
+    if (strcmp(set, "0") == 0 ||
+	strcasecmp(set, "zero") == 0)
+	searchdepth = NE_DEPTH_ZERO;
+    else if (strcasecmp(set, "1") == 0 ||
+	     strcasecmp(set, "one") == 0)
+	searchdepth = NE_DEPTH_ONE;
+    else
+	searchdepth = NE_DEPTH_INFINITE;
+}
+
+static void unset_searchdepth(const char *s)
+{
+    searchdepth = NE_DEPTH_INFINITE;
+}
+
+static void disp_searchdepth(void)
+{
+    if (searchdepth == NE_DEPTH_ZERO)
+	printf("0");
+    else if (searchdepth == NE_DEPTH_ONE)
+	printf("1");
+    else
+	printf("infinity");
+}
+
+static const struct option *find_option(const char *name)
+{
+    int n;
+    
+    for (n = 0; options[n].name != NULL; n++)
+        if (strcasecmp(options[n].name, name) == 0)
+            return &options[n];
+    
+    return NULL;
+}
+
 void execute_set(const char *opt, const char *newv)
 {
     if (opt == NULL) {
@@ -279,6 +350,7 @@ void execute_set(const char *opt, const char *newv)
 			printf("%s is a boolean option.\n", opt);
 		    } else {
 			*(int *)options[n].holder = 1;
+			setunset_expect100();
 		    }
 		    break;
 		case opt_string:
@@ -319,6 +391,7 @@ void execute_unset(const char *opt, const char *newv)
 		    printf("%s ia a boolean option.\n", opt);
 		} else {
 		    *(int *)options[n].holder = 0;
+		    setunset_expect100();
 		}
 		break;
 	    case opt_string:
@@ -341,6 +414,19 @@ void execute_unset(const char *opt, const char *newv)
     printf("Unknown option: %s.\n", opt);
 }
 
+void execute_describe(const char *name)
+{
+    const struct option *opt = find_option(name);
+
+    if (opt == NULL) {
+        printf("Option `%s' not known.\n", name);
+        return;
+    }
+
+    printf("Option `%s': %s\n", opt->name, opt->help);
+    if (opt->handle_help) puts(opt->handle_help);
+}
+
 void *get_option(enum option_id id)
 {
     int n;
@@ -355,13 +441,18 @@ void *get_option(enum option_id id)
 int get_bool_option(enum option_id id)
 {
     int *ret = get_option(id);
+    if (ret == NULL)
+	return 0;
     return *ret;
 }
 
 void set_bool_option(enum option_id id, int truth)
 {
     int *opt = get_option(id);
-    *opt = truth;
+    if (opt != NULL) {
+	*opt = truth;
+	setunset_expect100();
+    }
 }
 
 void set_option(enum option_id id, void *newval)
@@ -370,6 +461,7 @@ void set_option(enum option_id id, void *newval)
     for (n = 0; options[n].name != NULL; n++) {
 	if (options[n].id == id) {
 	    options[n].holder = newval;
+	    setunset_expect100();
 	    return;
 	}
     }
