@@ -26,8 +26,7 @@ enum option_id {
     opt_tolerant,
     opt_expect100,
     opt_editor,
-    opt_cert,
-    opt_certkey,
+    opt_clicert,
     opt_namespace,
     opt_quiet,
     opt_proxy,
@@ -39,14 +38,22 @@ enum option_id {
     opt_lockstore,
     opt_lockdepth,
     opt_lockscope,
-    opt_pager
+    opt_pager,
+   
+    opt_searchdepth,
+    opt_searchorder,
+    opt_searchdorder,
+    opt_searchall
 };
 
 extern int lockdepth; /* current lock depth setting. */
+extern int searchdepth; /* current search depth setting. */
 extern enum ne_lock_scope lockscope; /* current lock scope setting. */
 
 void execute_set( const char *opt, const char * );
 void execute_unset( const char *opt, const char * );
+/* Describe option of given name */
+void execute_describe(const char *name);
 
 void *get_option( enum option_id id );
 void set_option( enum option_id id, void *newval );
